﻿using System;
using System.Collections.Generic;
using System.Threading;

namespace TestProject
{
    /// <summary>
    /// 테스트 태스크
    /// </summary>
    public class TestTask
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 스레드
        /// </summary>
        private Thread thread;

        /// <summary>
        /// 완료 여부
        /// </summary>
        private bool isCompleted;

        /// <summary>
        /// 연속 액션 리스트
        /// </summary>
        private List<Action> continuationActionList = new List<Action>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 완료 여부 - IsCompleted

        /// <summary>
        /// 완료 여부
        /// </summary>
        public bool IsCompleted
        {
            get
            {
                return this.isCompleted;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TestTask(action)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="action">액션</param>
        public TestTask(Action action)
        {
            this.thread = new Thread
            (
                (ThreadStart)(() =>
                {
                    action();

                    this.isCompleted = true;

                    foreach(Action continuation in this.continuationActionList)
                    {
                        continuation();
                    }
                }
                )
            );

            this.thread.Start();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 연속 추가하기 - AddContinuation(action)

        /// <summary>
        /// 연속 추가하기
        /// </summary>
        /// <param name="action">액션</param>
        public void AddContinuation(Action action)
        {
            this.continuationActionList.Add(action);
        }

        #endregion
        #region 대기자 구하기 - GetAwaiter()

        /// <summary>
        /// 대기자 구하기
        /// </summary>
        /// <returns>대기자</returns>
        public TestTaskAwaiter GetAwaiter()
        {
            TestTaskAwaiter awaiter = new TestTaskAwaiter(this);

            return awaiter;
        }

        #endregion
    }
}